/**
 * @author p3ga5e
 */
 
THREE.Ms3dJoint = function(config) 
{
	for(var p in config)	
		this[p] = config[p];
	this.matrix = new THREE.Matrix4();
	
	function searchKeyFrames (time, keyframes)
	{
		for(var i=0;i<keyframes.length;i++)
		{
			if(time==keyframes[i].time)
				return [keyframes[i]];
			else if(time<keyframes[i].time)
			{
				if(i-1>=0)
					return[keyframes[i-1],keyframes[i]];
				else
					return[keyframes[i]];
			}
		}
		return [keyframes[keyframes.length-1]];
	}
	
	
	this.updateMatrix = function () 
	{
		this.matrix.identity();
		//this.matrix.setPosition( this.pos );
		this.matrix.setRotationFromEuler( this.rot );
		this.matrix.n14 = this.pos.x;
		this.matrix.n24 = this.pos.y;
		this.matrix.n34 = this.pos.z;
		
		if(arguments.length==1)
		{
			var time = arguments[0];
			var rotKeyFrames = searchKeyFrames(time,this.RotKeyFrames);
			var posKeyFrames = searchKeyFrames(time,this.PosKeyFrames);
			var pos,rot;
			if(posKeyFrames.length==1)
				pos = posKeyFrames[0].pos;
			else
			{	
				pos = new THREE.Vector3();
				var scale = ( time - posKeyFrames[0].time ) / ( posKeyFrames[1].time - posKeyFrames[0].time );
				// check scale error
				if ( scale < 0 || scale > 1 ) {

					console.log( "THREE.Ms3dJoint.updateMatrix: Warning! Scale out of bounds:" + scale  );
					scale = scale < 0 ? 0 : 1;
				}
				
				pos.x = posKeyFrames[0].pos.x + ( posKeyFrames[1].pos.x - posKeyFrames[0].pos.x ) * scale;
				pos.y = posKeyFrames[0].pos.y + ( posKeyFrames[1].pos.y - posKeyFrames[0].pos.y ) * scale;
				pos.z = posKeyFrames[0].pos.z + ( posKeyFrames[1].pos.z - posKeyFrames[0].pos.z ) * scale;
			}
			
			if(rotKeyFrames.length==1)
				rot = rotKeyFrames[0].rot;
			else
			{	
				rot = new THREE.Vector3();
				var scale = ( time - rotKeyFrames[0].time ) / ( rotKeyFrames[1].time - rotKeyFrames[0].time );
				// check scale error
				if ( scale < 0 || scale > 1 ) {

					console.log( "THREE.Ms3dJoint.updateMatrix: Warning! Scale out of bounds:" + scale  );
					scale = scale < 0 ? 0 : 1;
				}
				
				rot.x = rotKeyFrames[0].rot.x + ( rotKeyFrames[1].rot.x - rotKeyFrames[0].rot.x ) * scale;
				rot.y = rotKeyFrames[0].rot.y + ( rotKeyFrames[1].rot.y - rotKeyFrames[0].rot.y ) * scale;
				rot.z = rotKeyFrames[0].rot.z + ( rotKeyFrames[1].rot.z - rotKeyFrames[0].rot.z ) * scale;
			}	
			
			var frameMatrix = new THREE.Matrix4();
			frameMatrix.setRotationFromEuler(rot );
			frameMatrix.n14 = pos.x;
			frameMatrix.n24 = pos.y;
			frameMatrix.n34 = pos.z;
			
			this.matrix.multiplySelf(frameMatrix);
		}
		
		
	};
	
	this.getBoneMatrix = function () 
	{
		var mat = new THREE.Matrix4();
		if(!this.parent)
			mat.copy(this.matrix);
		else 
			mat.multiply(this.parent.getBoneMatrix(),this.matrix);
		
		return mat;
		//return this.matrix.clone();
	};
	
	
	
};


