/**
 * @author p3ga5e
 */

THREE.Ms3dLoader = function ( showStatus ) {

	THREE.Loader.call( this, showStatus );

};

THREE.Ms3dLoader.prototype = new THREE.Loader();
THREE.Ms3dLoader.prototype.constructor = THREE.Ms3dLoader;
THREE.Ms3dLoader.prototype.supr = THREE.Loader.prototype;


// Load a Milkshape3D Model fileVersion 1.3 and 1.4 is supported.
// Only Version 1.3 and 1.4 is supported.
//  - parameters
//		- url (required)
//		- callback (required)
//		- texturePath (optional: if not specified, textures will be assumed to be in the same folder as JS model file)

THREE.Ms3dLoader.prototype.load = function( url, callback, texturePath) {

	texturePath = texturePath ? texturePath : this.extractUrlbase( url );
	
	var callbackProgress = this.showProgress ? THREE.Loader.prototype.updateProgress : null;

	this.onLoadStart();

	this.loadAjaxBuffers( url, callback, texturePath, callbackProgress );

};

THREE.Ms3dLoader.prototype.loadAjaxBuffers = function(  url, callback, texturePath, callbackProgress ) {

	var xhr = new XMLHttpRequest();

	var length = 0;

	xhr.onreadystatechange = function() {

		if ( xhr.readyState == 4 ) {

			if ( xhr.status == 200 || xhr.status == 0 ) {

				THREE.Ms3dLoader.prototype.createBinModel( xhr.response, url, callback, texturePath);

			} else {

				console.error( "Couldn't load [" + url + "] [" + xhr.status + "]" );

			}

		} else if ( xhr.readyState == 3 ) {

			if ( callbackProgress ) {

				if ( length == 0 ) {

					length = xhr.getResponseHeader( "Content-Length" );

				}

				callbackProgress( { total: length, loaded: xhr.responseText.length } );

			}

		} else if ( xhr.readyState == 2 ) {

			length = xhr.getResponseHeader( "Content-Length" );

		}

	};

	xhr.open( "GET", url, true );
	xhr.responseType = "arraybuffer";
	xhr.send( null );
};


// Binary AJAX parser
THREE.Ms3dLoader.prototype.createBinModel = function ( data, url, callback, texturePath ) 
{
	var bs =  new BinaryStream(data);
	var materials = [];
	var animation = null;
	var Model = function ( ) {
		
		THREE.Geometry.call( this );
		this.metadata = {}
		this.metadata.signature = bs.readString(10);
		if(this.metadata.signature != "MS3D000000")
			throw new Error("Ms3dLoader : file '"+url+"' dont look like as an Milkshape3D Model file!"); 
		this.metadata.version = bs.readUInt32();
		if(this.metadata.version!=4)
			throw new Error("Ms3dLoader : file '"+url+"' only Version 1.3 and 1.4 is supported.");
		
		this.metadata.nvertices = bs.readUInt16();
		console.log("nvertices : " + this.metadata.nvertices);
		this.metadata.skinning=true;
		var bonesIds = []; 
		for(var i=0;i<this.metadata.nvertices;i++)
		{
			bs.skip(1);
			var vertex = new THREE.Vector3( bs.readSingle(), bs.readSingle(), bs.readSingle() );
			var boneId = bs.readInt8();
			if(boneId<0) this.metadata.skinning=false;
			if(this.metadata.skinning)
			{
				bonesIds.push(boneId);
			}
			bs.skip(1);
			this.vertices.push( new THREE.Vertex( vertex  ) );
		}
		if(this.metadata.skinning)
		{
			this.boneIndices = bonesIds;
			this.skinIndices = [];
			this.skinWeights = [];
		}
		this.metadata.ntriangles = bs.readUInt16();
		console.log("ntriangles : " + this.metadata.ntriangles);
		for(var i=0;i<this.metadata.ntriangles;i++)
		{
			bs.skip(2);
			var v1 = bs.readUInt16();
			var v2 = bs.readUInt16();
			var v3 = bs.readUInt16();
			var n1 = new THREE.Vector3( bs.readSingle(), bs.readSingle(), bs.readSingle() );
			var n2 = new THREE.Vector3( bs.readSingle(), bs.readSingle(), bs.readSingle() );
			var n3 = new THREE.Vector3( bs.readSingle(), bs.readSingle(), bs.readSingle() );
			var face = new THREE.Face3( v1, v2, v3, [ n1, n2, n3 ] );
			face.materialIndex = 0;
			var s = [ bs.readSingle(), bs.readSingle(), bs.readSingle()];
			var t = [ bs.readSingle(), bs.readSingle(), bs.readSingle()];
			var uv1 = new THREE.UV( s[0], t[0] );
			var uv2 = new THREE.UV( s[1], t[1] );
			var uv3 = new THREE.UV( s[2], t[2] );
			
			bs.skip(2);
			
			this.faces.push( face );
			this.faceVertexUvs[ 0 ].push( [ uv1, uv2, uv3 ] );
		}
		
		this.metadata.ngroups = bs.readUInt16();
		console.log("ngroups : " + this.metadata.ngroups);
		var matidx =[];
		for(var i=0;i<this.metadata.ngroups;i++)
		{
			bs.skip(33);
			var n = bs.readUInt16();
			bs.skip(n*2);
			matidx.push(bs.readInt8());
		}
		this.metadata.nmaterials = bs.readUInt16();
		console.log("nmaterials : " + this.metadata.nmaterials);
		for(var i=0;i<this.metadata.nmaterials;i++)
		{
			
			var material = 
			{
				name	 		: bs.readZeroTerminatedString(32), 
				ambient			: new THREE.Color(),   
				diffuse			: new THREE.Color(), 
				specular		: new THREE.Color(), 
				emissive		: new THREE.Color()
			};

			material.ambient.setRGB(bs.readSingle(), bs.readSingle(), bs.readSingle());bs.skip(4);
			material.diffuse.setRGB(bs.readSingle(), bs.readSingle(), bs.readSingle());bs.skip(4);
			material.specular.setRGB(bs.readSingle(), bs.readSingle(), bs.readSingle());bs.skip(4);
			material.emissive.setRGB(bs.readSingle(), bs.readSingle(), bs.readSingle());bs.skip(4);
			material.shininess = bs.readSingle();
			material.transparency = bs.readSingle();
			bs.skip(1); // skip Mode
			var file = bs.readZeroTerminatedString(128);
			if(file.length>0)
			{
				var idx = file.lastIndexOf('/');
				if(idx>0)
					file = file.substr(idx+1);
				material.texture = THREE.ImageUtils.loadTexture(texturePath+file);
			}
			file = bs.readZeroTerminatedString(128);
			if(file.length>0)
			{
				var idx = file.lastIndexOf('/');
				if(idx>0)
					file = file.substr(idx+1);
				material.alphamap = THREE.ImageUtils.loadTexture(texturePath+file);
			}
			materials.push(material);
		}
		
		if(this.metadata.skinning)
		{
			var Rad2Deg = 180.0 / Math.PI;
			this.Joints = [];
			var  boneIndex = {};
			this.metadata.animation = 
			{	fps  		: bs.readSingle(),
				currentTime : bs.readSingle(),
				totalFrames : bs.readInt32()
			};
			//animation.length = animation.TotalFrames/animation.fps;
			var numBones = bs.readUInt16();
			for(var i=0;i<numBones;i++)
			{	
				bs.skip(1);
				var bone = 
				{	name : bs.readZeroTerminatedString(32),
					parentName : bs.readZeroTerminatedString(32),
					rot : new THREE.Vector3(bs.readSingle(), bs.readSingle(), bs.readSingle()),
					pos : new THREE.Vector3(bs.readSingle(), bs.readSingle(), bs.readSingle()),
					RotKeyFrames : [],
					PosKeyFrames : []
				};
				boneIndex[bone.name] = i;
				
				//console.log(bone.name,bone.rotq,bone.pos);
				var numKeyFramesRot = bs.readUInt16();
				var numKeyFramesPos = bs.readUInt16();
				for(var j=0;j<numKeyFramesRot;j++)
				{
					bone.RotKeyFrames.push(
					{ 	time	: bs.readSingle(),
						rot		: new THREE.Vector3(bs.readSingle(), bs.readSingle(), bs.readSingle())
					});
				}
				for(var j=0;j<numKeyFramesPos;j++)
				{
					bone.PosKeyFrames.push(
					{ 	time	: bs.readSingle(),
						pos		: new THREE.Vector3(bs.readSingle(), bs.readSingle(), bs.readSingle())
					});
				}
				this.Joints.push(new THREE.Ms3dJoint(bone));
			}
			for(var i=0;i<numBones;i++)
			{
				var bone = this.Joints[i];
				if(bone.parentName.length!=0)
					bone.parent = this.Joints[boneIndex[bone.parentName]];	
			}
			
		}
		this.computeCentroids();
		this.computeFaceNormals();
		
	};
	
	Model.prototype = new THREE.Geometry();
	Model.prototype.constructor = Model;

	callback( new Model(), materials, animation );
};

